import { isPlatformBrowser, DOCUMENT } from '@angular/common';
import * as i0 from '@angular/core';
import { PLATFORM_ID, Injectable, Inject, NgModule } from '@angular/core';
import { BehaviorSubject, Subject } from 'rxjs';

// @dynamic
class WindowService {
    constructor(document, 
    // tslint:disable-next-line: ban-types
    platformId) {
        this.document = document;
        this.platformId = platformId;
        this.widthSubject$ = new BehaviorSubject(1200);
        this.scrollingUpSubject$ = new BehaviorSubject(true);
        this.currentScrollPositionSubject$ = new BehaviorSubject(0);
        /**
         * Observable to get the window-width, defaults to 1200 when no window is defined
         */
        this.width$ = this.widthSubject$.asObservable();
        /**
         * Observable to track when the scroll has ended
         */
        this.scrollingUp$ = this.scrollingUpSubject$.asObservable();
        /**
         * Observable of the current scroll position after the scroll has ended
         */
        this.currentScrollPosition$ = this.currentScrollPositionSubject$.asObservable();
        /**
         * Current scroll position after the scroll has ended
         */
        this.currentScrollPosition = 0;
        if (this.isBrowser() && this.hasDocument()) {
            this.window = this.document.defaultView;
            this.document.addEventListener('scroll', this.handleContentScroll.bind(this));
            this.widthSubject$.next(this.window.innerWidth);
            this.window.addEventListener('resize', () => {
                if (this.window.innerWidth && this.widthSubject$.getValue()) {
                    this.widthSubject$.next(this.window.innerWidth);
                }
            });
        }
    }
    /**
     * Scrolls to the provided position of the page
     *
     * @param offset - Offset to which we want to scroll, scrolls to top when no offset is provided
     */
    scrollTo(offset = 0) {
        if (!this.window) {
            return;
        }
        this.window.scrollTo(0, offset);
    }
    /**
     * Returns whether there is a document present
     */
    hasDocument() {
        return !!this.document;
    }
    /**
     * Returns whether the current platform is a browser
     */
    isBrowser() {
        return isPlatformBrowser(this.platformId);
    }
    handleContentScroll() {
        if (window.pageYOffset > this.currentScrollPosition) {
            this.scrollingUpSubject$.next(false);
        }
        else {
            this.scrollingUpSubject$.next(true);
        }
        this.currentScrollPosition = window.pageYOffset < 0 ? 0 : window.pageYOffset;
        this.currentScrollPositionSubject$.next(this.currentScrollPosition);
    }
}
WindowService.ɵfac = function WindowService_Factory(t) { return new (t || WindowService)(i0.ɵɵinject(DOCUMENT), i0.ɵɵinject(PLATFORM_ID)); };
WindowService.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: WindowService, factory: WindowService.ɵfac, providedIn: 'root' });
(function () { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(WindowService, [{
        type: Injectable,
        args: [{
                providedIn: 'root',
            }]
    }], function () { return [{ type: Document, decorators: [{
                type: Inject,
                args: [DOCUMENT]
            }] }, { type: undefined, decorators: [{
                type: Inject,
                args: [PLATFORM_ID]
            }] }]; }, null); })();

/*
* This windowMock constant return an object with a selected spy.
* The reason why the spy is not provided by default is for this mock
* to work on both Jest and Jasmine based test-suites.
*
* example:
* windowMock(jasmine.createSpy());
* */
const windowMock = (spy) => ({
    addEventListener: spy,
    defaultView: {
        scrollTo: () => null,
        innerWidth: 720,
        addEventListener: spy,
    },
});
/*
* Provide a spy for your testing framework because the windowMock requires it.
*
* The reason why the spy is not provided by default is for this mock
* to work on both Jest and Jasmine based test-suites.
*
* example:
* windowServiceMock(jasmine.createSpy(), 1440);
* */
const windowServiceMock = (spy, width = 1200) => ({
    width: new BehaviorSubject(width),
    window: windowMock(spy),
    scrollTo: () => null,
    hasDocument: () => true,
    isBrowser: () => true,
});

class SubscriptionService {
    constructor() {
        this.destroyed$ = new Subject();
    }
    ngOnDestroy() {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    }
}
SubscriptionService.ɵfac = function SubscriptionService_Factory(t) { return new (t || SubscriptionService)(); };
SubscriptionService.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SubscriptionService, factory: SubscriptionService.ɵfac });
(function () { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SubscriptionService, [{
        type: Injectable
    }], null, null); })();

class UtilsModule {
}
UtilsModule.ɵfac = function UtilsModule_Factory(t) { return new (t || UtilsModule)(); };
UtilsModule.ɵmod = /*@__PURE__*/ i0.ɵɵdefineNgModule({ type: UtilsModule });
UtilsModule.ɵinj = /*@__PURE__*/ i0.ɵɵdefineInjector({ providers: [WindowService], imports: [[]] });
(function () { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(UtilsModule, [{
        type: NgModule,
        args: [{
                declarations: [],
                imports: [],
                providers: [WindowService],
                exports: [],
            }]
    }], null, null); })();

/*
 * Public API Surface of utils
 */

/**
 * Generated bundle index. Do not edit.
 */

export { SubscriptionService, UtilsModule, WindowService, windowMock, windowServiceMock };
//# sourceMappingURL=studiohyperdrive-ngx-utils.js.map
